;Written for AT90S2313 at 10MHz

;RS232-Interface
;===============
;PortB is Output, PortD 2...6 is Input.
;To read input, send same byte twice (byte must not be 0) in a
;short time.
;To set output, just send the byte once and wait some time.

;Version 1.0 finished 21.11.2003

;-----------------------------------------------------------------

.include "2313def.inc"
.def	temp	= r16
.def	last	= r17
.def	count	= r18
.def	outp	= r19

;-----------------------------------------------------------------

.cseg
.org $0000
	rjmp	start
.org $0007
	push	temp
	in	temp,	SREG
	push	temp
	in	temp,	UDR		;get received byte
	cp	temp,	last		;if the same as last byte:
	breq	getinp			;  send input byte
	mov	last,	temp		;remember the byte
	ldi	outp,	1		;not outputed yet
return:
	pop	temp
	out	SREG,	temp
	pop	temp
	reti				;return to main program
getinp:
	cpi	temp,	0		;if received byte was 0:
	breq	return			;  return (no action)
	in	temp,	PIND		;get input from PortD
	lsr	temp			;shift right twice
	lsr	temp			;  (coz PD0+PD1 are UART!)
	out	UDR,	temp		;send inputed byte
	clr	last
	clr	outp
	clr	count
	rjmp	return			;return
		
;-----------------------------------------------------------------

start:
	ldi	temp,	low(RAMEND)	;set stackpointer
	out	SPL,	temp
	ldi	temp,	0b11111111	;PB0-7=Out
	out	DDRB,	temp
	ldi	temp,	0b00000010	;PD0/2-6=In, PD1=Out
	out	DDRD,	temp
	ldi	temp,	0b11111101	;Pullups on
	out	PORTD,	temp
	ldi	temp,	0b10011000	;set UART control
	out	UCR,	temp
	ldi	temp,	64		;set BAUD rate
	out	UBRR,	temp		; --> 9600 @ 10MHz
	sei				;Interrupts on
loop:
	inc	count			;increment counter
	brne	ok			;if overflow:
	ldi	temp,	0		;  if something to output:
	cpse	outp,	temp
	out	PORTB,	last		;    output it!
	ldi	outp,	0		;  nothing(more)to output
	ldi	last,	0		;  reset last byte
ok:
	clr	temp,	0		;wait some time
wait:
	inc	temp
	brne	wait
	rjmp	loop			;endless loop
